@@ -8,8 +8,8 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION          = '0.49';
-our $WSDL_DATE        = '2007-10-29';
+our $VERSION          = '0.59';
+our $WSDL_DATE        = '2009-07-01';
 our $Locale           = 'us';
 our @CANNED_RESPONSES = ();
 our $IS_CANNED        = 0;
@@ -21,12 +21,15 @@ use Data::Dumper;
 use URI;
 use Log::Log4perl qw(:easy get_logger);
 use Time::HiRes qw(usleep gettimeofday tv_interval);
+use Digest::SHA qw(hmac_sha256_base64);
+use URI::Escape qw(uri_escape);
 
 # Each key represents a search() type, and each value indicates which
 # Net::Amazon::Request:: class to use to handle it.
 use constant SEARCH_TYPE_CLASS_MAP => {
     actor        => 'Actor',
     artist       => 'Artist',
+    all          => 'All',
     author       => 'Author',
     asin         => 'ASIN',
     blended      => 'Blended',
@@ -57,6 +60,10 @@ sub new {
         die "Mandatory paramter 'token' not defined";
     }
 
+    if(! exists $options{secret_key}) {
+        die "Mandatory paramter 'secret_key' not defined";
+    }
+
     my $self = {
         strict         => 1,
         response_dump  => 0,
@@ -155,6 +162,13 @@ sub request {
             'Version'        => $WSDL_DATE,
             map { $_, $params{$_} } sort keys %params,
         );
+	
+        # Signed requests will have different URLs, which breaks caching.
+        # Get a cachable URL before signing the request.
+        my $url_cachablestr = $url->as_string;
+
+        # New signature for >=2009-03-31. Do not alter URL after this!
+        $url = $self->_sign_request($url) if exists $self->{secret_key};
 
         DEBUG(sub { "request: params = " . Dumper(\%params) . "\n"});
 
@@ -162,7 +176,7 @@ sub request {
 
         DEBUG(sub { "urlstr=" . $urlstr });
 
-        my $xml = fetch_url($self, $urlstr, $res);
+        my $xml = fetch_url($self, $urlstr, $url_cachablestr, $res);
 
         if(!defined $xml) {
             return $res;
@@ -220,7 +234,7 @@ sub request {
 ##################################################
 sub fetch_url {
 ##################################################
-    my($self, $url, $res) = @_;
+    my($self, $url, $url_cachablestr, $res) = @_;
 
     my $max_retries = 2;
 
@@ -233,7 +247,7 @@ sub fetch_url {
     }
 
     if(exists $self->{cache}) {
-        my $resp = $self->{cache}->get($url);
+        my $resp = $self->{cache}->get($url_cachablestr);
         if(defined $resp) {
             INFO("Serving from cache");
             return $resp;
@@ -300,7 +314,7 @@ sub fetch_url {
     }
 
     if(exists $self->{cache}) {
-        $self->{cache}->set($url, $resp->content());
+        $self->{cache}->set($url_cachablestr, $resp->content());
     }
 
     return $resp->content();
@@ -464,7 +478,7 @@ sub is_page_error {
                 push @errors, $e->{Message};
             }
             # multiple errors, set arrary ref
-            $self->messages( @errors );
+            $self->messages( \@errors );
         } else {
             # single error, create array
             $self->messages( [ $errref->{Error}->{Message} ] );
@@ -598,6 +612,30 @@ sub _make_request {
     return $self->request($req);
 }
 
+# $self->_sign_request( URI )
+#
+# Takes a URI object that corresponds to a Net::Amazon::Request
+# adds the required Timestamp and Signature parameters, and returns it
+# See http://docs.amazonwebservices.com/AWSECommerceService/2009-03-31/DG/Query_QueryAuth.html
+sub _sign_request {
+    my ($self,$uri) = @_;
+    return $uri unless exists $self->{secret_key};
+    # This assumes no duplicated keys. Safe assumption?
+    my %query = $uri->query_form;
+    my @now = gmtime;
+    $query{Timestamp} ||= sprintf('%04d-%02d-%02dT%02d:%02d:%02dZ',$now[5]+1900,$now[4]+1,@now[3,2,1,0]);
+    my $qstring = join '&', map {"$_=". uri_escape($query{$_},"^A-Za-z0-9\-_.~")} sort keys %query;
+    # Use chr(10), not "\n" which varies by platform
+    my $signme = join chr(10),"GET",$uri->host,$uri->path,$qstring;
+    my $sig = hmac_sha256_base64($signme, $self->{secret_key});
+    # Digest does not properly pad b64 strings
+    $sig .= '=' while length($sig) % 4;
+    $sig = uri_escape($sig,"^A-Za-z0-9\-_.~");
+    $qstring .= "&Signature=$sig";
+    $uri->query( $qstring );
+    return $uri;
+}
+
 1;
 
 __END__
@@ -610,7 +648,9 @@ Net::Amazon - Framework for accessing amazon.com via REST
 
   use Net::Amazon;
 
-  my $ua = Net::Amazon->new(token => 'YOUR_AMZN_TOKEN');
+  my $ua = Net::Amazon->new(
+	token      => 'YOUR_AMZN_TOKEN',
+	secret_key => 'YOUR_AMZN_SECRET_KEY');
 
     # Get a request object
   my $response = $ua->search(asin => '0201360683');
@@ -634,7 +674,8 @@ C<Net::Amazon> works very much like C<LWP>: First you define a useragent
 like
 
   my $ua = Net::Amazon->new(
-      token     => 'YOUR_AMZN_TOKEN',
+      token      => 'YOUR_AMZN_TOKEN',
+      secret_key => 'YOUR_AMZN_SECRET_KEY',
       max_pages => 3,
   );
 
@@ -731,7 +772,6 @@ for details.
 Initiate a search for all items made by a given music label. Can return many
 results. See Net::Amazon::Request::MusicLabel for details.
 
-
 =item C<< $ua->search(publisher => "o'reilly") >>
 
 Initiate a search for all items made by a given publisher. Can return many
@@ -858,6 +898,11 @@ Music search by Actor, mandatory parameter "actor". Can return many results.
 Music search by Artist, mandatory parameter C<artist>.
 Can return many results.
 
+=item Net::Amazon::Request::All
+
+'All' search on a keyword, mandatory parameter C<all>.
+Can return many results.
+
 =item Net::Amazon::Request::Author
 
 Music search by Author, mandatory parameter "author". Can return many results.
@@ -949,11 +994,15 @@ The convenient C<search()> method just does these two steps in one.
 
 =over 4
 
-=item $ua = Net::Amazon->new(token => $token, ...)
+=item $ua = Net::Amazon->new(token => $token, secret_key => $key, ...)
 
 Create a new Net::Amazon useragent. C<$token> is the value of 
 the mandatory Amazon developer's token, which can be obtained from
-L<http://amazon.com/soap>. 
+L<http://aws.amazon.com>.
+
+By 2009-08-15 Amazon will require that all requests be signed with an Amazon
+assigned Secret Key.  The Secret Key can be obtained from
+L<http://aws.amazon.com>.
 
 Additional optional parameters:
 
@@ -988,7 +1037,9 @@ Sends a request to the Amazon web service. C<$request> is of a
 C<Net::Amazon::Request::*> type and C<$response> will be of the 
 corresponding C<Net::Amazon::Response::*> type.
 
-=back
+=back 
+
+=back 
 
 =head2 Modes
 
@@ -1037,7 +1088,8 @@ Here's a full-fledged example doing a artist search:
         unless defined $ARGV[0];
 
     my $ua = Net::Amazon->new(
-        token       => 'YOUR_AMZN_TOKEN',
+        token      => 'YOUR_AMZN_TOKEN',
+        secret_key => 'YOUR_AMZN_SECRET_KEY',
     );
 
     my $req = Net::Amazon::Request::Artist->new(
@@ -1062,7 +1114,8 @@ And here's one displaying someone's wishlist:
         "(use 1XL5DWOUFMFVJ as an example)\n" unless $ARGV[0];
 
     my $ua = Net::Amazon->new(
-        token       => 'YOUR_AMZN_TOKEN',
+        token      => 'YOUR_AMZN_TOKEN',
+        secret_key => 'YOUR_AMZN_SECRET_KEY',
     );
 
     my $req = Net::Amazon::Request::Wishlist->new(
@@ -1080,7 +1133,7 @@ And here's one displaying someone's wishlist:
 
 DETAILS
         Net::Amazon is based on Amazon Web Services version 4, and uses
-        WSDL version 2007-01-17.
+        WSDL version 2009-07-01.
 
 =head1 CACHING
 
@@ -1103,6 +1156,7 @@ cache responses for 30 minutes:
 
     my $ua = Net::Amazon->new(
         token       => 'YOUR_AMZN_TOKEN',
+        secret_key  => 'YOUR_AMZN_SECRET_KEY',
         cache       => $cache,
     );
 
@@ -1144,7 +1198,11 @@ pass a user agent instance to Net::Amazon's constructor:
     use LWP::UserAgent;
 
     my $ua = LWP::UserAgent->new();
-    my $na = Net::Amazon->new(ua => $ua, token => 'YOUR_AMZN_TOKEN');
+    my $na = Net::Amazon->new(
+	ua         => $ua, 
+	token      => 'YOUR_AMZN_TOKEN',
+        secret_key => 'YOUR_AMZN_SECRET_KEY',
+    );
     # ...
 
 This way, you can configure C<$ua> up front before Net::Amazon will use it.
@@ -1218,9 +1276,9 @@ Write a new Net::Amazon::Request::XYZ package, start with this template
         if(!exists $options{XYZ_option}) {
             die "Mandatory parameter 'XYZ_option' not defined";
         }
-    
+
         my $self = $class->SUPER::new(%options);
-    
+
         bless $self, $class;   # reconsecrate
     }
 
@@ -1237,9 +1295,9 @@ and add documentation. Then, create a new Net::Amazon::Response::XYZ module:
     sub new {
     ##############################
         my($class, %options) = @_;
-    
+
         my $self = $class->SUPER::new(%options);
-    
+
         bless $self, $class;   # reconsecrate
     }
 
@@ -1381,6 +1439,13 @@ use C<Net::Amazon>, want to report a bug or just participate in its
 development, please send a message to the mailing 
 list net-amazon-devel@lists.sourceforge.net
 
+The source code has moved from sourceforge.net to github.com.  The git URL is
+
+    git://github.com/boumenot/p5-Net-Amazon.git
+
+The hope is that github.com makes collaboration much easier, and git is 
+a much more modern SCM tool.
+
 =head1 AUTHOR
 
 Mike Schilli, E<lt>na@perlmeister.comE<gt> (Please contact me via the mailing list: net-amazon-devel@lists.sourceforge.net )
@@ -1410,10 +1475,12 @@ Contributors (thanks y'all!):
     Steve Rushe <steve@deeden.co.uk>
     Tatsuhiko Miyagawa <miyagawa@livedoor.jp>
     Tony Bowden <tony@kasei.com>
+    Vince Veselosky
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2003, 2004 by Mike Schilli E<lt>na@perlmeister.comE<gt>
+Copyright 2007-2009 by Christopher Boumenot E<lt>boumenot+na@gmail.comE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
